use raylib::prelude::*;
use std::io::{self, BufRead};
use std::sync::{Arc, RwLock};
use std::time::Duration;

const POSITIVE: Color = Color::new(0, 0, 200, 255);
const NEGATIVE: Color = Color::new(175, 0, 0, 255);
const SCALE: i32 = 10;
const SIZE: i32 = 50;
const BOUNDS: i32 = SIZE * SCALE;

fn main() {
    let input = io::stdin().lock().lines().next().unwrap().unwrap();
    let mut has_reached_basement = false;

    let (mut rl, thread) = raylib::init()
        .size(BOUNDS, BOUNDS)
        .title("Advent of Code 2015, Day 1 Visualisation")
        .build();

    let shared_sum = Arc::new(RwLock::new(0));
    let writer_sum = Arc::clone(&shared_sum);

    std::thread::spawn(move || {
        for (count, char) in input.chars().enumerate() {
            match char {
                '(' => {
                    *writer_sum.write().unwrap() += 1;
                }
                ')' => {
                    let mut writer = writer_sum.write().unwrap();
                    *writer -= 1;
                    if !has_reached_basement && *writer == -1 {
                        println!("reached basement on the {}th char", count + 1);
                        has_reached_basement = true;
                    }
                }
                _ => (),
            }
            std::thread::sleep(Duration::from_millis(2));
        }
        println!("end count: {}", writer_sum.write().unwrap());
    });

    while !rl.window_should_close() {
        let mut d = rl.begin_drawing(&thread);
        d.clear_background(Color::WHITE);

        let sum = shared_sum.read().unwrap();
        let mut sum = *sum;
        let mut x = 0;
        let mut y = 0;
        let mut draw_colour = POSITIVE;
        if sum < 0 {
            draw_colour = NEGATIVE;
            sum = -sum;
        }

        for i in 0..sum {
            d.draw_rectangle(x * SCALE, y * SCALE, SCALE, SCALE, draw_colour);
            x += 1;
            if i != 0 && i % 50 == 0 {
                x = 0;
                y += 1;
            }
        }
    }
}
